#!/usr/bin/env python

import argparse
import os
import sys
import numpy as np
import pyshellspec

def read_lc(lfile):
    """
    """
    data = np.loadtxt(lfile, dtype=str)

    names = data[:,0]
    pbands = data[:,1]
    errs = data[:,2].astype('float64')

    return names, pbands, errs

def read_if(ifile):
    """

    :param ifile:
    :return:
    """
    return np.loadtxt(ifile, dtype=str)


# constants
ra = 360 * 18/24. + 50 / 60. + 04.79525 / 3600.
dec = 33 + 21 / 60. + 45.6100 / 3600.
#dir_data = '/home/mira/a/betalyr_MYTEST/data_20171126'
dir_data = '/scratch/mira/a/betalyr_MYTEST/data_20171126'

def main():
    """
    Main loop
    :return:
    """
    # read the data
    obs = []

    # first photometry
    dir = os.path.join(dir_data, 'photometry')
    infile = os.path.join(dir, 'lc.lis')
    names, pbands, errs = read_lc(infile)
    for i in range(0, len(names)):
        print names[i]
        obs.append(pyshellspec.LCData(filename=os.path.join(dir, names[i]),
                                      passband=pbands[i],
                                      global_error=errs[i],
                                      delimiter=','))

    # data from MIRC
    dir = os.path.join(dir_data, 'mirc2017')
    infile = os.path.join(dir, 'mirc.ascii.lis')
    names = read_if(infile)
    for i in range(0, len(names)):
        print names[i]
        obs.append(pyshellspec.IFData(filename=os.path.join(dir, names[i]),
                                      location='chara',
                                      ra=ra, dec=dec, 
                                      format='ascii', weight_vis2=0.5, weight_t3amp=0.5))
    # data from NPOI
    dir = os.path.join(dir_data, 'npoi')
    infile = os.path.join(dir, 'npoi.ascii.lis')
    names = read_if(infile)
    for i in range(0, len(names)):
        print names[i]
        obs.append(pyshellspec.IFData(filename=os.path.join(dir, names[i]),
                                      location='npoi',
                                      ra=ra, dec=dec,
                                      format='ascii', exclude_t3amp=True))
    # data from VEGA
    dir = os.path.join(dir_data, 'vega')
    infile = os.path.join(dir, 'vega.ascii.lis')
    d = read_if(infile)
    fnames = d[:, 0]
    names = d[:, 1]
    for i in range(0, len(names)):
        print names[i]
        obs.append(pyshellspec.IFData(filename=os.path.join(dir, fnames[i]),
                                      location='chara',
                                      ra=ra, dec=dec,
                                      format='ascii'))
        obs[-1].set_filename(names[i])

    # construct data class
    data = pyshellspec.Data(obs)

    # construct the model
    cenobj = pyshellspec.CentralObject()
    compan = pyshellspec.Companion()
    disk = pyshellspec.Disk()
    nebula = pyshellspec.Nebula()
    orbit = pyshellspec.Orbit()
    objs = [cenobj, compan, disk, nebula, orbit]
    model = pyshellspec.Model(objects=objs)

    # construct the Interface
    itf = pyshellspec.Interface(model=model, data=data, ncpu=4, image_size=180,
				if_phase_precision=3, lc_phase_precision=2, 
				if_ew_precision=7, lc_ew_precision=9,
				shellspec_template="template.in", use_offset=True)
    # set grid resolution
    itf.set_parameter('steps', value=1.0)
    itf.set_parameter('stepf', value=1.0)
    itf.set_parameter('stepsz', value=1.0)
    itf.set_parameter('stepfz', value=1.0)
    #itf.set_parameter('steps', value=10.0)
    #itf.set_parameter('stepf', value=10.0)
    #itf.set_parameter('stepsz', value=10.0)
    #itf.set_parameter('stepfz', value=10.0)

    # set fitted parameter
    itf.set_parameter('inebl', value=4)  # nebula = flared protoplanetary disk
    itf.set_parameter('itnb', value=3)  # power-law

    itf.set_parameter('ichemc', value=1)
    itf.set_parameter('ielnd', value=1)
    itf.set_parameter('ithom', value=0)
    itf.set_parameter('irayl', value=0)
    itf.set_parameter('imie', value=0)
    itf.set_parameter('imiepf', value=0)

    itf.set_parameter('routnb'  , value=31.09856399261549    , fitted=True, vmin=26., vmax=35.)
    itf.set_parameter('hinvnb'  , value=3.458852964026885    , fitted=True, vmin=1.0, vmax=9.0)
    itf.set_parameter('tinvnb'  , value=8.8                  , fitted=True, vmin=1.0, vmax=9.0)
    itf.set_parameter('hwindnb' , value=4.990967922867704    , fitted=True, vmin=3.0, vmax=9.0)
    itf.set_parameter('tempnb'  , value=30526.40161263414    , fitted=True, vmin=23000., vmax=34000.)
    itf.set_parameter('densnb'  , value=2.433623239928714e-06, fitted=True, vmin=2e-8, vmax=5e-6)
    itf.set_parameter('edennb'  , value=-2.464776138715449   , fitted=True, vmin=-3.5, vmax=-0.8)
    itf.set_parameter('etmpnb'  , value=-1.010523400177847   , fitted=True, vmin=-1.1, vmax=-0.70)
    itf.set_parameter('hmulnb'  , value=5.383774794819662    , fitted=True, vmin=1.0, vmax=15.0)
    itf.set_parameter('dinc'    , value=93.4956864306707     , fitted=True, vmin=91., vmax=97.)
    itf.set_parameter('omega_an', value=253.880178438755     , fitted=True, vmin=252., vmax=255.)
    itf.set_parameter('dd'      , value=322.1679207427022    , fitted=True, vmin=305., vmax=330.)

    itf.set_parameter('aneb'    , value=9.0          , fitted=False, vmin=3.0, vmax=9.0)
    itf.set_parameter('asini'   , value=58.19        , fitted=False, vmin=53., vmax=63.)
    itf.set_parameter('tempcp'  , value=13300        , fitted=False, vmin=12000., vmax=14600.)

    itf.set_parameter('iline', value=1)

    # compute one fit
#    itf.compute_chi2(verbose=True)
#    itf.run_fit(fitter='sp_diff_evol', tol=1e-2, maxiter=10000)
    itf.run_fit(fitter='nlopt_nelder_mead', ftol=1e-6, maxiter=1000)

    itf.write_iterations()
    itf.set_model_to_shellspec()
    itf.write_template('final.in')
    itf.write_model()

    print "Note: fit.py ended successfully."
    sys.exit(0) 

    files = [] 
    for o in obs:
        files.append(o.get_filename().split('/')[-1])
    for f in list(set(files)):
        itf.plot_comparison(filename=f)

if __name__ == '__main__':
    main()


